@@ -3,10 +3,6 @@ from rath.links.parsing import ParsingLink
 
 
 class ShrinkingLink(ParsingLink):
-    def __init__(self, file_path: str) -> None:
-        super().__init__()
-        self.file_path = file_path
-
     def parse(self, operation: Operation) -> Operation:
         shrinked_variables = {
             key: var.shrink()
