@@ -1,8 +1,5 @@
 import asyncio
-from dataclasses import dataclass
-
-from pydantic import BaseModel
-from koil.decorators import koilable
+from koil.composition import KoiledModel
 from rath.errors import NotConnectedError
 from rath.links.base import TerminatingLink
 from typing import (
@@ -19,9 +16,7 @@ from contextvars import ContextVar
 current_rath = ContextVar("current_rath")
 
 
-@koilable(add_connectors=True)
-@dataclass
-class Rath:
+class Rath(KoiledModel):
     link: Optional[TerminatingLink] = None
     set_context: bool = True
 
@@ -94,7 +89,6 @@ class Rath:
             yield res
 
     async def __aenter__(self):
-        self.link(self)
         if self.set_context:
             current_rath.set(self)
         await self.link.__aenter__()
@@ -104,9 +98,8 @@ class Rath:
     async def __aexit__(self, *args, **kwargs):
         await self.link.__aexit__(*args, **kwargs)
         self._connected = False
-        self.link(None)
         if self.set_context:
             current_rath.set(None)
 
-    def __enter__(self) -> "Rath":
-        ...
+    class Config:
+        underscore_attrs_are_private = True
